import flwr as fl
import tensorflow as tf
import sys

# Function to load datasets based on the provided argument
def load_data(dataset_name):
    if dataset_name == "cifar10":
        (x_train, y_train), (x_test, y_test) = tf.keras.datasets.cifar10.load_data()
        x_train = x_train.astype('float32') / 255.0
        x_test = x_test.astype('float32') / 255.0
        num_classes = 10  # CIFAR-10 has 10 classes
        input_shape = (32, 32, 3)
    elif dataset_name == "mnist":
        (x_train, y_train), (x_test, y_test) = tf.keras.datasets.mnist.load_data()
        x_train = x_train.reshape(-1, 28, 28, 1).astype('float32') / 255.0  # Add channel dimension
        x_test = x_test.reshape(-1, 28, 28, 1).astype('float32') / 255.0  # Add channel dimension
        num_classes = 10  # MNIST has 10 classes
        input_shape = (28, 28, 1)
    elif dataset_name == "cifar100":
        (x_train, y_train), (x_test, y_test) = tf.keras.datasets.cifar100.load_data()
        x_train = x_train.astype('float32') / 255.0
        x_test = x_test.astype('float32') / 255.0
        num_classes = 100  # CIFAR-100 has 100 classes
        input_shape = (32, 32, 3)
    else:
        raise ValueError("Dataset not supported: choose 'cifar10', 'mnist', or 'cifar100'")
    
    return (x_train, y_train), (x_test, y_test), num_classes, input_shape

# Default dataset to CIFAR-10
dataset_name = "cifar10"  # Default dataset if no argument is passed
if len(sys.argv) > 1:
    dataset_name = sys.argv[1]

# Load the dataset
(x_train, y_train), (x_test, y_test), num_classes, input_shape = load_data(dataset_name)

# Define the model (MobileNetV2 for this example)
model = tf.keras.applications.MobileNetV2(input_shape=input_shape, classes=num_classes, weights=None)
model.compile("adam", "sparse_categorical_crossentropy", metrics=["accuracy"])

# Define Flower client
class CifarClient(fl.client.NumPyClient):
    def get_parameters(self, config):
        return model.get_weights()

    def fit(self, parameters, config):
        model.set_weights(parameters)
        model.fit(x_train, y_train, epochs=1, batch_size=32, steps_per_epoch=3)
        return model.get_weights(), len(x_train), {}

    def evaluate(self, parameters, config):
        model.set_weights(parameters)
        loss, accuracy = model.evaluate(x_test, y_test)
        return loss, len(x_test), {"accuracy": float(accuracy)}

# Start Flower client
if len(sys.argv) < 3:
    fl.client.start_client(server_address="127.0.0.1:8080", 
                           client=CifarClient(),
                           max_retries=5,
                           max_wait_time=10) #in seconds
else:
    fl.client.start_client(server_address=sys.argv[2], 
                           client=CifarClient(),
                           max_retries=5,
                           max_wait_time=10)
