# python3 kill.py --config ./config_instances_lille.json --user mdo --serverkey "python3 /home/mdo/eflwr/Flower_v1/server_1.py" --clientkey "/home/mdo/eflwr/Flower_v1/client_1.py"
#!/usr/bin/env python3

import os
import subprocess
import json
import argparse
from pathlib import Path

def parse_args():
    parser = argparse.ArgumentParser(description="Kill background jobs by keyword and user")
    parser.add_argument("--config", required=True, help="Path to the config JSON file")
    parser.add_argument("--user", required=True, help="Username to filter processes")
    parser.add_argument("--serverkey", required=True, help="Keyword to identify server process")
    parser.add_argument("--clientkey", required=True, help="Keyword to identify client processes")
    return parser.parse_args()

def check_and_kill_process(host, keyword, user):
    """
    Check for processes matching the keyword on the host and kill them (only for the given user).
    """
    try:
        print(f"\n========== Checking processes on {host} for user '{user}' ==========")

        current_pid = str(os.getpid())

        # Command to find processes by user and keyword
        grep_command = f"oarsh {host} pgrep -u {user} -fa '{keyword}'"

        result = subprocess.run(grep_command, shell=True, text=True, capture_output=True)

        if result.returncode == 0 and result.stdout.strip():
            process_lines = result.stdout.strip().split("\n")
            for line in process_lines:
                pid, process_name = line.split(" ", 1)

                if pid == current_pid:
                    print(f"Skipping current script process (PID={pid})")
                    continue

                print(f"Found process: PID={pid}, Process={process_name}")
                
                kill_command = f"oarsh {host} kill -9 {pid}"
                subprocess.run(kill_command, shell=True)
                print(f"Killed process {pid} on {host}")
        else:
            print("No matching processes found.")
    except Exception as e:
        print(f"Error on host {host}: {e}")

def main():
    args = parse_args()

    # Load config
    config_path = Path(args.config).resolve()
    with open(config_path, "r") as file:
        config = json.load(file)["instances"]["1"]

    # Get server IP
    server_ip = config["server"]["ip"]

    # Get client list
    client_details = [client["ip"] for client in config["clients"]]

    # Kill server processes
    check_and_kill_process(server_ip, args.serverkey, args.user)

    # Kill client processes
    for client_ip in client_details:
        if client_ip:
            check_and_kill_process(client_ip, args.clientkey, args.user)

    print("\n========== Process Management Completed ==========")

if __name__ == "__main__":
    main()
